
/*---------------------------------------------
		
	MatNN library : Test App 4 (cpp)

		This shows the copy constructor working.
		(4 different instances of a net are drawn,
		that were created by copy constructors)

		It will show the assignment operator
		working aswell, when/if i decide its right
		to assign neural nets.

		keys : r = random neural nets
			   a = decrease response
			   d = increase response

	By Matt Barnett

		(see References)	* See matnn.h for more information
	
---------------------------------------------*/

//basic glut and math stuff for camera etc

#include <ctime>
#include <cstdio>
#include <windows.h>
#include <GL/glut.h>

#include "../../../imath/imath.h"
#include "../../../matnn/matnn.h"

GLfloat grey[] = {0.2,0.2,0.2,1.0};
GLfloat above_origin[] = {0.0,0.0,3.0,1.0};
GLfloat white[] = {1.0,1.0,1.0,1.0};
GLfloat	shiny[] = {50.0};

#define WINDOW_SIZE			1024		//
#define	SCALE_MOTION		100			//scale down by

int x,y,mx,my,xs,ys;
int oldmx = 0;
int oldmy = 0;

enum modetype { MODE_ROTATE,MODE_MOVE,MODE_CAM };
modetype mode;
iquaternion4 camquat;

//globals for testing here

using MatLayeredNeuralNetLib::NeuralNet;
float resp = 1.0f;
float inputs[6];
float outputs[3];

NeuralNet *n0 = meNew NeuralNet(6,3,4,5,inputs,outputs);	//on the heap, so that we can delete it showing that n1 is copyied properly (no crosslinks)
NeuralNet n1 = *n0;											//calls the copy ctor , (allowed)

//------------------------
	
void drawaxes()
{
  glDisable(GL_LIGHTING);

  glBegin(GL_LINES);
  glColor3f(1.0,0.0,0.0);glVertex3f(-0.5,0.0,0.0);glVertex3f(1.2,0.0,0.0);
  glColor3f(0.0,1.0,0.0);glVertex3f(0.0,-0.5,0.0);glVertex3f(0.0,1.2,0.0);
  glColor3f(0.0,0.0,1.0);glVertex3f(0.0,0.0,-0.5);glVertex3f(0.0,0.0,1.2);
  glEnd();
}

void DrawBrainWave(NeuralNet &in,float floorsize,float floorstep,int x,int y)
{

	float i,j;

	float fftime = (float)GetTickCount() / 1500;
	float bp1 = 0.0;
	float bp2 = 1.0;
	float bp3 = 0.0;

	glPushMatrix();	
	glTranslatef((float)x,(float)y,0.0);


	for (i=0;i<floorsize;i+=floorstep)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for (j=0;j<floorsize;j+=floorstep)
		{
			
				inputs[0] = j;
				inputs[1] = i;
				inputs[2] = fftime;	
				inputs[3] = bp1;
				inputs[4] = bp2;
				inputs[5] = bp3;
				in.RunNN();

				glColor3f(outputs[0],outputs[1],outputs[2]);
				glVertex3f(i,j,outputs[2]);	
				
				inputs[0] = j+floorstep;
				inputs[1] = i+floorstep;
				inputs[2] = fftime;	
				
				inputs[3] = bp1;
				inputs[4] = bp2;
				inputs[5] = bp3;
				in.RunNN();

				glColor3f(outputs[0],outputs[1],outputs[2]);
				glVertex3f(i+floorstep,j+floorstep,outputs[2]);
		}
		glEnd();
	}


	drawaxes();
	glPopMatrix();
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	imatrix16 rm(camquat);
	glTranslatef(0.0,0.0,-3.0);
	glMultMatrixf(rm.m);

	glTranslatef((float)-1,(float)-1,0.0);

	//instert drawing here 
	DrawBrainWave(n1,1.5f,0.08f,0,0);
	DrawBrainWave(n2,1.5f,0.08f,2,2);
	DrawBrainWave(n3,1.5f,0.08f,0,2);
	DrawBrainWave(n4,1.5f,0.08f,2,0);

	glutSwapBuffers();
	glutPostRedisplay();
}



void reshape(int w, int h)
{
  glViewport(0,0,(GLsizei) w,(GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90,(float)w/h,0.01,100.0);
}



void motion(int x, int y)
{
	mx = x;my = y;

	xs = mx - oldmx;
	ys = oldmy - my;

	oldmx = mx;
	oldmy = my;

	iquaternion4 multquat;

	if (mode == MODE_ROTATE)
	{	
	
	}
	else if (mode == MODE_MOVE)
	{

	}
	else if (mode == MODE_CAM)
	{
		multquat.makeEulerRotation( ivector3( (float)ys/SCALE_MOTION,-(float)xs/SCALE_MOTION,0.0 ) );
		camquat.multiply(multquat);
	}
}


void mouse(int button,int state,int x,int y)
{
	oldmx = x;
	oldmy = y;

	if (button == GLUT_LEFT_BUTTON)
		mode = MODE_ROTATE;

	if (button == GLUT_RIGHT_BUTTON)
		mode = MODE_MOVE;

	
	if (button == GLUT_MIDDLE_BUTTON)
		mode = MODE_CAM;
}

void init(void)
{

	srand( (unsigned)time( NULL ) );	


	glEnable(GL_DEPTH_TEST);
	//glEnable(GL_LIGHTING);
	//glEnable(GL_LIGHT0);
	//glLightfv(GL_LIGHT0,GL_POSITION,above_origin);
	
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	//glMaterialfv(GL_FRONT,GL_DIFFUSE,grey);
	//glMaterialfv(GL_FRONT,GL_SPECULAR,white);
	//glMaterialfv(GL_FRONT,GL_SHININESS,shiny);

	//glEnable(GL_POLYGON_OFFSET_FILL);
	//glPolygonOffset(0.1,0.1);
	//glDepthFunc(GL_LEQUAL);
	//glEnable(GL_DEPTH_TEST);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//glEnable(GL_BLEND);
	//glEnable(GL_POLYGON_SMOOTH);
	//glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	
	//here we delete n2

	meDelete n0;

	glPointSize(4.0f);

	//wont compile (not allowed)
	//n1 = n2;

	inputs[0] = 0;
	inputs[1] = 0;

	//n1.SetResponse(resp);
}

void key(GLubyte key, int x, int y)
{
  switch (key)
    {
			case 'a':resp -= 0.05f;printf("response %f\n",resp);n1.SetResponse(resp);n2.SetResponse(resp);n3.SetResponse(resp);n4.SetResponse(resp);
			break;

			case 'd':resp += 0.05f;printf("response %f\n",resp);n1.SetResponse(resp);n2.SetResponse(resp);n3.SetResponse(resp);n4.SetResponse(resp);
			
			break;

			case 'r':im_random_seed_time();n1.Randomize();n2.Randomize();n3.Randomize();n4.Randomize();

			break;

			case '1':

			break;

			case '!':

			break;


			case 'l':

			break;


			case 'q':exit(0);
			break;
    }
	
}

int main(int argc, char **argv)
{
	meInitMemoryCheck();

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(WINDOW_SIZE,WINDOW_SIZE);
	glutInitWindowPosition(100,10);
	glutCreateWindow("Matts test");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutMotionFunc(motion);
	glutMouseFunc(mouse);

	init();
	glutMainLoop();

	return 0;
}